<?php
## PhpPatcher class
# @author legolas558
# @version 0.2
# Licensed under GNU General Public License (GPL)
#
# Facility to merge unified diff files
# First use Merge() and then ApplyPatch() to commit changes
# files will be created, updated and/or deleted
# using the auto_commit option files will be automatically commited

define('_PHPP_INVALID_INPUT', 'Invalid input');
define('_PHPP_UNEXPECTED_EOF', 'Unexpected end of file');
define('_PHPP_UNEXPECTED_ADD_LINE', 'Unexpected add line at line %d');
define('_PHPP_UNEXPECTED_REMOVE_LINE', 'Unexpected remove line at line %d');
define('_PHPP_INVALID_DIFF', 'Invalid unified diff block');
define('_PHPP_FAILED_VERIFY', 'Failed source verification of file %s at line %d');
define('_PHPP_INVALID_OFFSET', 'Failed source verification of file %s: expected at least %d lines');

class	PhpPatcher {
	
	var $root;
	var $msg;
	var $destinations = array();
	var $removals = array();
	var $newline = "\n";
	var $_auto_commit;
	var $_adone;
	
	function PhpPatcher($root_path, $auto_commit = 0) {
		// if you specify a root path all paths will be intended as relative to it (and not written, too)
		$this->root = $root_path;
		$this->_auto_commit = $auto_commit;
		$this->_adone = 0;
	}
	
	function _get_source($src) {
/*		header('Content-Type: text/plain');
		echo 'GET SOURCE: '.$src;
		echo '<pre>';var_dump($this->_linesplit($rd));echo '</pre>';	*/
			
		return $this->_linesplit(file_get_contents($src));
	}

	// retrieve the modified lines array, if any
	function &_get_destin($dst, &$source_lines) {
		if (isset($this->destinations[$dst]))
			return $this->destinations[$dst];
		$this->destinations[$dst] = $source_lines;
		return $this->destinations[$dst];
	}
	
	// separate CR or CRLF lines
	function &_linesplit(&$data) {
		$lines = preg_split('/(\r\n)|(\r)|(\n)/', $data);
		return $lines;
	}
	
	function Merge($udiff) {
		// (1) Split input into lines
		$lines = $this->_linesplit($udiff);
		if (!isset($lines)) {
			$this->msg = _PHPP_INVALID_INPUT;
			return false;
		}
		unset($udiff);
	
		$line = current($lines);
		do {
			if (strlen($line)<5)
				continue;
			// start recognition when a new diff block is found
			if (substr($line, 0, 4)!='--- ')
				continue;
			$p = strpos($line, "\t", 4);
			if ($p===false)	$p = strlen($line);
			$src = $this->root.substr($line, 4, $p-4);
			$line = next($lines);
			if ($line===FALSE) {
				$this->msg = _PHPP_UNEXPECTED_EOF;
				return false;
			}
			if (substr($line, 0, 4)!='+++ ') {
				$this->msg = _PHPP_INVALID_DIFF;
				return false;
			}
			$p = strpos($line, "\t", 4);
			if ($p===false)	$p = strlen($line);
			$dst = $this->root.(string)substr($line, 4, $p-4);
			
			$line = next($lines);
			if ($line===FALSE) {
				$this->msg = _PHPP_UNEXPECTED_EOF;
				return false;
			}
			
			$this->_auto_commit();
			
			$done=0;
			if (file_exists($src))
				$src_lines = $this->_get_source($src);
			else $src_lines = array();
			while (preg_match('/@@ -(\\d+)(,(\\d+))?\\s+\\+(\\d+)(,(\\d+))?\\s+@@($)/A', $line, $m)) {
			
				if ($m[3]==='')
					$src_size = 1;
				else $src_size = (int)$m[3];
				if ($m[6]==='')
					$dst_size = 1;
				else $dst_size = (int)$m[6];
				if (!$this->_apply_diff($lines, $src, $src_lines, $dst,
									(int)$m[1], $src_size, (int)$m[4],
									$dst_size))
					return false;
				$done++;
				$line = next($lines);
				if ($line === FALSE)
					break 2;
			}
			if ($done==0) {
				$this->msg = _PHPP_INVALID_DIFF;
				return false;
			}
			// do not eat the last line
			prev($lines);
			
		} while (FALSE !== ($line = next($lines)));
		
		//NOTE: only destination files are cached
		return true;
	}
	
	function ClearCache() {
		$this->destinations = array();
		$this->removals = array();
	}
	
	function _auto_commit() {
		// auto-commit previous patches (if option active) to lower memory usage
		if (!$this->_auto_commit)	return;
		if (count($this->destinations)>=$this->_auto_commit) {
			$this->ApplyPatch();
		}
	}
	
	function ApplyPatch() {
		if (empty($this->destinations))
			return $this->_adone;
		$files = array_keys($this->destinations);
		foreach($files as $file) {
			$f = @fopen($file, 'wb');
			if ($f===false)
				continue;
			fwrite($f, implode("\n", $this->destinations[$file]));
			fclose($f);
			$this->_adone++;
		}
		foreach($this->removals as $file) {
			if (@unlink($file))
				$this->_adone++;
		}
		$this->destinations = array(); // clear the destinations cache
		$this->removals = array();
		return $this->_adone;
	}
	
	function _apply_diff(&$lines, $src_fname, &$src_lines, $dst, $src_line, $src_size, $dst_line, $dst_size) {
		$src_line--;
		$dst_line--;
		$line = next($lines);
		if ($line === false) {
			$this->msg = _PHPP_UNEXPECTED_EOF;
			return false;
		}
		$source = array();		// source lines (old file)
		$destin = array();		// new lines (new file)
		$src_left = $src_size;
		$dst_left = $dst_size;
		
		$dst_lines =& $this->_get_destin($dst, $src_lines);
		do {
			if (!strlen($line)) {
				$source[] = '';
				$destin[] = '';
				$src_left--;
				$dst_left--;
				continue;
			}
			if ($line[0]=='-') {
				if ($src_left==0) {
					$this->msg = sprintf(_PHPP_UNEXPECTED_REMOVE_LINE, key($lines));
					return false;
				}
				$source[] = (string)substr($line, 1);
				$src_left--;
			} else if ($line[0]=='+') {
				if ($dst_left==0) {
					$this->msg = sprintf(_PHPP_UNEXPECTED_ADD_LINE, key($lines));
					return false;
				}
				$destin[] = (string)substr($line, 1);
				$dst_left--;
			} else {
				if (!strlen($line))
					$line = '';
				else if ($line[0]=='\\') {
					if ($line=='\\ No newline at end of file')
						continue;
				} else
					$line = (string)substr($line, 1);
				$source[] = $line;
				$destin[] = $line;
				$src_left--;
				$dst_left--;
			}
			
			if (($src_left==0) && ($dst_left==0)) {
				// now apply the patch, finally!
				if ($dst_size>0) {
					if ($src_size>0) {
						$src_bottom=$src_line+count($source);
						$dst_bottom=$dst_line+count($destin);
						
						if ($src_bottom>count($src_lines)) {
							$this->msg = sprintf(_PHPP_INVALID_OFFSET, $src_fname, $src_bottom);
							return false;
						}
						
						for ($l=$src_line;$l<$src_bottom;$l++) {
							if ($src_lines[$l]!==$source[$l-$src_line]) {
/*							
							header('Content-Type: text/plain');
//								echo '<pre>'.$src.':';var_dump($src_lines);echo '</pre>';die;
								echo 'Source has '.count($src_lines).' but line '.$l.' was requested<br>';
								echo '<pre>SRC LINE '.$l.' IS:';var_dump($src_lines[$l]);echo '</pre>';
								echo '<pre>SRC LINE '.($l-$src_line).' OUGHT TO BE:';var_dump($source[$l-$src_line]);echo '</pre>';
								echo '<pre>';var_dump($source);echo '</pre>';die;
								die;	*/
								
								$this->msg = sprintf(_PHPP_FAILED_VERIFY, $src_fname, $l);
								return false;
							}
						}
						$span = count($source);
					} else 	// new files creation
						$span = count($src_lines);
					// patches the destination array
					array_splice($dst_lines, $dst_line, $span, $destin);

				} else
					$this->removals[] = $src_fname;
				
				return true;
			}
		} while (FALSE !== ($line = next($lines)));

		$this->msg = _PHPP_UNEXPECTED_EOF;
		return false;
	}
	
	function Diff(&$latest, &$udiff) {
		$this->msg = 'Not available';
		return false;
	}

}

?>